@@ -55,9 +55,10 @@ import Text.XML.HXT.Arrow.XmlState.TypeDefs
 {- |
 the main document input filter
 
-this filter can be configured by a list of configuration options, a value of type 'Text.XML.HXT.XmlState.TypeDefs.SysConfig'
+this filter can be configured by a list of configuration options,
+a value of type 'Text.XML.HXT.XmlState.TypeDefs.SysConfig'
 
-for all available options see module 'Text.XML.HXT.XmlState.SystemConfig'
+for all available options see module 'Text.XML.HXT.Arrow.XmlState.SystemConfig'
 
 - @withValidate yes\/no@ :
   switch on\/off DTD validation. Only for XML parsed documents, not for HTML parsing.
@@ -72,7 +73,8 @@ for all available options see module 'Text.XML.HXT.XmlState.SystemConfig'
 - @withSubstHTMLEntities yes\/no@ :
   switch on\/off entity substitution for general entities defined in HTML validation.
   Default is @no@.
-  Switching this option on and the validation and substDTDEntities off can lead to faster parsing, in that case
+  Switching this option on and the validation and substDTDEntities off can lead to faster parsing,
+  in that case
   reading the DTD documents is not longer necessary, HTML general entities are still substituted.
   Only used with XML parsed documents, not with HTML parsing.
 
@@ -86,27 +88,27 @@ for all available options see module 'Text.XML.HXT.XmlState.SystemConfig'
 - @withCheckNamespaces yes\/no@ :
   Switch on\/off namespace propagation and checking
 
-- @withInputEncoding <encoding-spec>@ :
+- @withInputEncoding \<encoding-spec\>@ :
   Set default encoding.
 
 - @withTagSoup@ :
   use light weight and lazy parser based on tagsoup lib.
   This is only available when package hxt-tagsoup is installed and
-  'Text.XML.HXT.TagSoup' is imported
+  the source contains an @import Text.XML.HXT.TagSoup@.
 
-- @withRelaxNG <schema.rng>@ :
+- @withRelaxNG \<schema.rng\>@ :
   validate document with Relax NG, the parameter is for the schema URI.
   This implies using XML parser, no validation against DTD, and canonicalisation.
 
-- @withCurl [<curl-option>...]@ :
+- @withCurl [\<curl-option\>...]@ :
   Use the libCurl binding for HTTP access.
   This is only available when package hxt-curl is installed and
-  'Text.XML.HXT.Curl' is imported
+  the source contains an @import Text.XML.HXT.Curl@.
                    
-- @withHTTP [<http-option>...]@ :
+- @withHTTP [\<http-option\>...]@ :
   Use the Haskell HTTP package for HTTP access.
   This is only available when package hxt-http is installed and
-  'Text.XML.HXT.HTTP' is imported
+  the source contains an @import Text.XML.HXT.HTTP@.
 
 examples:
 
@@ -222,7 +224,8 @@ readDocument'' src
           getSysVar (theParseByMimeType   .&&&.
                      theParseHTML         .&&&.
                      theAcceptedMimeTypes .&&&.
-                     theRelaxValidate
+                     theRelaxValidate     .&&&.
+                     theXmlSchemaValidate
                     )
         )
       )
@@ -276,7 +279,12 @@ readDocument'' src
             processDoc' mimeType options
           )
 
-    processDoc' mimeType (parseByMimeType, (parseHtml, (acceptedMimeTypes, validateWithRelax)))
+    processDoc' mimeType ( parseByMimeType
+                         , ( parseHtml
+                           , ( acceptedMimeTypes
+                             , ( validateWithRelax
+                               , validateWithXmlSchema
+                               ))))
         = ( if isAcceptedMimeType acceptedMimeTypes mimeType
             then ( processNoneEmptyDoc
                    ( ( parse $< getSysVar (theValidate              .&&&.
@@ -307,7 +315,7 @@ readDocument'' src
                                                         )
                               )
                               >>>
-                              relax
+                              relaxOrXmlSchema
                             )
                        else this
                      )
@@ -339,7 +347,13 @@ readDocument'' src
                                             (mi == mis || mis == "*")
                                           )
                                           || r
-        parse (validate, (substDTD, (substHTML, (removeNoneXml, (withTagSoup', withExpat')))))
+        parse ( validate
+              , ( substDTD
+                , ( substHTML
+                  , ( removeNoneXml
+                    , ( withTagSoup'
+                      , withExpat'
+                      )))))
             | not isXmlOrHtml           = if removeNoneXml
                                           then replaceChildren none             -- don't parse, if mime type is not XML nor HTML
                                           else this                             -- but remove contents when option is set
@@ -352,7 +366,11 @@ readDocument'' src
 
             | isXml                     = if withExpat'
                                           then parseXmlDocumentWithExpat
-                                          else parseXmlDocument validate substDTD substHTML validateWithRelax
+                                          else parseXmlDocument
+                                               validate
+                                               substDTD
+                                               substHTML
+                                               validateWithRelax
                                                                                 -- parse as XML
             | otherwise                 = this                                  -- suppress warning
 
@@ -363,25 +381,35 @@ readDocument'' src
 
             | withNamespaces
               ||
-              validateWithRelax         = propagateAndValidateNamespaces        -- RelaxNG requires correct namespaces
+              validateWithRelax
+              ||
+              validateWithXmlSchema
+                                        = propagateAndValidateNamespaces        -- RelaxNG and XML Schema require correct namespaces
 
             | otherwise                 = this
 
         canonicalize (preserveCmt, (canonicalize', withTagSoup'))
             | withTagSoup'              = this                                  -- tagsoup already removes redundant stuff
-            | validateWithRelax         = canonicalizeAllNodes
+            | validateWithRelax
+              ||
+              validateWithXmlSchema     = canonicalizeAllNodes                  -- no comments in schema validation
+
             | canonicalize'
               &&
               preserveCmt               = canonicalizeForXPath
             | canonicalize'             = canonicalizeAllNodes
             | otherwise                 = this
 
-        relax
+        relaxOrXmlSchema
+            | validateWithXmlSchema     = withoutUserState $< getSysVar theXmlSchemaValidator
             | validateWithRelax         = withoutUserState $< getSysVar theRelaxValidator
             | otherwise                 = this
 
         whitespace (removeWS, withTagSoup')
-            | removeWS
+            | ( removeWS
+                ||
+                validateWithXmlSchema                                           -- XML Schema does not like WS
+              )
               &&
               not withTagSoup'          = removeDocWhiteSpace                   -- tagsoup already removes whitespace
             | otherwise                 = this
@@ -417,15 +445,16 @@ readFromDocument config
 -- read a document that is stored in a normal Haskell String
 --
 -- the same function as readDocument, but the parameter forms the input.
--- All options available for 'readDocument' are applicable for readString.
+-- All options available for 'readDocument' are applicable for readString,
+-- except input encoding options.
 --
--- Default encoding: No encoding is done, the String argument is taken as Unicode string
+-- Encoding: No decoding is done, the String argument is taken as Unicode string
+-- All decoding must be done before calling readString, even if the
+-- XML document contains an encoding spec.
 
 readString :: SysConfigList -> String -> IOStateArrow s b XmlTree
 readString config content
-    = readDocument
-      (withInputEncoding unicodeString : config)
-      (stringProtocol ++ content)
+    = readDocument config (stringProtocol ++ content)
 
 -- ------------------------------------------------------------
 
