@@ -181,6 +181,13 @@ instance Eq QName where
                                   &&                    -- and prefixes are not set or they are equal
                                   px1 == px2
 
+instance Ord QName where
+  compare (QN lp1 px1 ns1) (QN lp2 px2 ns2)
+      | isNullXName ns1 && isNullXName ns2              -- no namespaces set: px is significant
+          = compare (px1, lp1) (px2, lp2)
+      | otherwise                                       -- namespace aware cmp: ns is significant, px is irrelevant
+          = compare (lp1, ns1) (lp2, ns2)
+
 instance NFData  QName
 instance WNFData QName
 
