@@ -57,51 +57,53 @@ module Text.XML.HXT.Parser.XmlParsec
     )
 where
 
-import Text.ParserCombinators.Parsec                    ( runParser
-                                                        , (<?>), (<|>)
-                                                        , char
-                                                        , string
-                                                        , eof
-                                                        , between
-                                                        , many
-                                                        , many1
-                                                        , notFollowedBy
-                                                        , option
-                                                        , try
-                                                        , unexpected
-                                                        , getPosition
-                                                        , getInput
-                                                        , sourceName
-                                                        )
+import Control.Applicative                      ( (<$>) )
 
-import Text.XML.HXT.DOM.ShowXml                         ( xshow
-                                                        )
+import Text.ParserCombinators.Parsec            ( runParser
+                                                , (<?>), (<|>)
+                                                , char
+                                                , string
+                                                , eof
+                                                , between
+                                                , many
+                                                , many1
+                                                , notFollowedBy
+                                                , option
+                                                , try
+                                                , unexpected
+                                                , getPosition
+                                                , getInput
+                                                , sourceName
+                                                )
+
+import Text.XML.HXT.DOM.ShowXml                 ( xshow
+                                                )
 import Text.XML.HXT.DOM.Interface
-import Text.XML.HXT.DOM.XmlNode                         ( mkElement'
-                                                        , mkAttr'
-                                                        , mkRoot'
-                                                        , mkDTDElem'
-                                                        , mkText'
-                                                        , mkCmt'
-                                                        , mkCdata'
-                                                        , mkError'
-                                                        , mkPi'
-                                                        , isText
-                                                        , isRoot
-                                                        , getText
-                                                        , getChildren
-                                                        , getAttrl
-                                                        , getAttrName
-                                                        , changeAttrl
-                                                        , mergeAttrl
-                                                        )
-import Text.XML.HXT.Parser.XmlCharParser                ( xmlChar
-                                                        , XParser
-                                                        , SimpleXParser
-                                                        , XPState
-                                                        , withNormNewline
-                                                        , withoutNormNewline
-                                                        )
+import Text.XML.HXT.DOM.XmlNode                 ( mkElement'
+                                                , mkAttr'
+                                                , mkRoot'
+                                                , mkDTDElem'
+                                                , mkText'
+                                                , mkCmt'
+                                                , mkCdata'
+                                                , mkError'
+                                                , mkPi'
+                                                , isText
+                                                , isRoot
+                                                , getText
+                                                , getChildren
+                                                , getAttrl
+                                                , getAttrName
+                                                , changeAttrl
+                                                , mergeAttrl
+                                                )
+import Text.XML.HXT.Parser.XmlCharParser        ( xmlChar
+                                                , XParser
+                                                , SimpleXParser
+                                                , XPState
+                                                , withNormNewline
+                                                , withoutNormNewline
+                                                )
 import qualified Text.XML.HXT.Parser.XmlTokenParser     as XT
 import qualified Text.XML.HXT.Parser.XmlDTDTokenParser  as XD
 
@@ -462,20 +464,22 @@ content
 
 content         :: XParser s XmlTrees
 content
-    = many $
-      ( do		-- parse markup but no closing tags
-        try ( XT.lt
-              >>
-              notFollowedBy (char '/')
-              >>
-              return ()
-            )
-	markup
+    = XT.mergeTextNodes <$>
+      many
+      ( ( do		-- parse markup but no closing tags
+          try ( XT.lt
+                >>
+                notFollowedBy (char '/')
+                >>
+                return ()
+              )
+	  markup
+        )
+        <|>
+        charData'
+        <|>
+        XT.referenceT
       )
-      <|>
-      charData'
-      <|>
-      XT.referenceT
     where
     markup
 	= element'
@@ -608,7 +612,8 @@ encodingDecl
 --
 -- the string parameter is parsed with the XML content parser.
 -- result is the list of trees or in case of an error a single element list with the
--- error message as node. No entity or character subtitution is done.
+-- error message as node. No entity or character subtitution is done here,
+-- but the XML parser can do this for the predefined XML or the char references for performance reasons
 --
 -- see also: 'parseXmlContent'
 
